import { styled } from "solid-styled-components";
import { Show, createSignal, onMount } from "solid-js";
import { vector2d } from "./vector";
import SignalBus from "../signal";

import Player from "./objects/player";
import Crystal from "./objects/crystal";
import TimerProvider from "./providers/timer";
import CooldownProvider from "./providers/cooldown";
import CooldownUI from "./ui/cooldown";

import "./objects/common.css";
import OverviewProvider from "./providers/overview";
import OverviewUI from "./ui/overview";
import SpawnFarm, { EnemiesTemplates } from "./objects/spawn-farm";

const StageContainer = styled("div")`
  width: 100vw;
  height: 100vh;

  position: relative;

  overflow: hidden;
`;

export default function Stage() {
  let container: HTMLDivElement = document.createElement("div");

  let stageSize = new vector2d(0, 0);
  let centerPosition = new vector2d(0, 0);
  let playerPosition = new vector2d(0, 0);

  let [ready, setReady] = createSignal(false);

  const enemiesSpawnSignal = new SignalBus();

  function initializeStage(element: HTMLDivElement) {
    stageSize = new vector2d(element.clientWidth, element.clientHeight);
    centerPosition = new vector2d(stageSize.x / 2, stageSize.y / 2);
    playerPosition = centerPosition.clone();

    setInterval(() => enemiesSpawnSignal.emit(), 1000)

    setReady(true);
  }

  onMount(() => initializeStage(container));

  return (
    <TimerProvider>
      <OverviewProvider>
        <CooldownProvider>
          <StageContainer class="stage" ref={container}>
            <Show when={ready()}>
              {/* Objects */}
              <Crystal size={128} position={centerPosition} />
              <Player size={64} position={playerPosition} />

              {/* Object Sets */}
              <SpawnFarm
                templates={EnemiesTemplates}
                signal={enemiesSpawnSignal}
                borderX={stageSize.x}
                borderY={stageSize.y}
              />

              {/* UI Elements */}
              <CooldownUI />
              <OverviewUI />
            </Show>
          </StageContainer>
        </CooldownProvider>
      </OverviewProvider>
    </TimerProvider>
  );
}
