import { createSignal, type Component, JSX, For, createMemo } from "solid-js";
import { vector2d } from "../vector";
import SignalBus from "../../signal";

import Attacker from "./enemies/attacker";
import { TimerContext, useTimer } from "../providers/timer";

export const EnemiesTemplates = [
  { element: Attacker, weight: 10 },
  { element: Attacker, weight: 10 },
];

export default function SpawnFarm({
  templates,
  signal,
  borderX,
  borderY,
}: {
  templates: { element: Component; weight: number }[];
  signal: SignalBus;
  borderX: number;
  borderY: number;
}) {
  const [objects, setObjects] = createSignal<JSX.Element[]>([]);

  const timer = useTimer();

  function createObject(template: Component, props?: any) {
    return (
      <TimerContext.Provider value={timer}>
        {template(props)}
      </TimerContext.Provider>
    );
  }

  function pickTemplate() {
    const totalWeight = templates.reduce((sum, { weight }) => sum + weight, 0);
    const randomNumber = Math.random() * totalWeight;

    let cumulativeWeight = 0;
    for (let i = 0; i < templates.length; i++) {
      cumulativeWeight += templates[i].weight;
      if (randomNumber < cumulativeWeight) {
        return templates[i].element;
      }
    }
  }

  function pickPosition() {
    return new vector2d(
      Math.floor(Math.random() * borderX),
      Math.floor(Math.random() * borderY)
    );
  }

  function generate() {
    const template = pickTemplate();
    if (template) {
      const object = createObject(template, { position: pickPosition() });
      setObjects(objects().concat([object]));
    }
  }

  signal.subscribe(() => generate());

  return <For each={objects()}>{(item) => item}</For>;
}
